package io.vaadinpush.issuereprod.util.base

import io.vaadinpush.issuereprod.util.extension.isNull
import org.apache.tika.mime.MimeTypes
import java.io.File


object FileHelper {


    private val jarLocationFile: File by lazy {
        File(this::class.java
            .protectionDomain
            .codeSource
            .location
            .toURI()
            .path)
    }

    private val developerResourcesLocation:String = "/Users/fluento/Downloads/minifyimage_files"

    object FolderName {
        const val caches: String = "caches"
        const val resources: String = "resources"
        const val frontendSessions: String = "frontendSessions"
        const val accounts: String = "accounts"
        const val images: String = "images"
        const val invoices: String = "invoices"
        const val subscriptions: String = "subscriptions"
        const val temp: String = "temp"
    }

    object Extensions {
        const val TXT: String = ".txt"
        const val JSON: String = ".json"
        const val PKCS12: String = ".p12"
        const val CRT_DER: String = ".der"
        const val PKCS8: String = ".p8"
        const val PKCS10: String = ".p10"
        const val CERT: String = ".cert"
    }

    object MimeType {
        const val TXT: String = "text/plain"
        const val JSON: String = "application/json"
        const val PKCS12: String = "application/x-pkcs12"
        const val PEM: String = "application/x-pem-file"
        const val CRT_DER: String =  "application/x-x509-ca-cert"
        const val CRT: String = "application/x-x509-user-cert"
        const val PKCS8: String = "application/pkcs8"
        const val PKCS10: String = "application/pkcs10"
        const val CERT: String = "application/pkix-cert"

        fun getMimeType(mimeType:String?):org.apache.tika.mime.MimeType?{
            if(mimeType.isNull()){
                return null
            }

            return try{
                MimeTypes.getDefaultMimeTypes().forName(mimeType)
            }catch (e:Exception){
                null
            }
        }

        fun getExtension(mimeType:String?):String?{
            if(mimeType.isNull()){
                return null
            }
            return try{
                getMimeType(mimeType)?.extension
           }catch (e:Exception){
             null
           }
        }

    }

    private val resourcesFolderPath: String by lazy {
//        "${jarLocationFile.absolutePath}/${FolderName.resources}"
        "${developerResourcesLocation}/${FolderName.resources}"
    }

    private val cachesFolderPath: String by lazy {
        "${jarLocationFile.absolutePath}/${FolderName.caches}"
    }

    fun getFolderTemp(createIfNotExists: Boolean = true): File? {
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.temp}",
            createIfNotExists
        )
    }

    fun getFolderTemp(temporaryResourceName:String,createIfNotExists: Boolean = true): File? {
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.temp}/${temporaryResourceName}",
            createIfNotExists
        )
    }

    fun getFolderForFrontendSession(frontendSessionId: String, createIfNotExists: Boolean = true): File? {
        if(frontendSessionId.isNull()){
            return null
        }
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.frontendSessions}/${frontendSessionId}",
            createIfNotExists
        )
    }

    fun getFolderForFrontendSessionImage(frontendSessionId: String,imageId:String, createIfNotExists: Boolean = true): File? {
        if(frontendSessionId.isNull() || imageId.isNull()){
            return null
        }
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.frontendSessions}/${imageId}",
            createIfNotExists
        )
    }

    fun getFolderForAccount(accountId: String, createIfNotExists: Boolean = true): File? {
        if(accountId.isNull()){
            return null
        }
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.accounts}/${accountId}",
            createIfNotExists
        )
    }

    fun getFolderForImage(accountId: String,imageId:String, createIfNotExists: Boolean = true): File? {
        if(accountId.isNull() || imageId.isNull()){
            return null
        }
        return createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.accounts}/${accountId}/${FolderName.images}/${imageId}",
            createIfNotExists
        )
    }

    fun getInputImage(accountId: String,imageId:String,fileNameWithExtension:String, createIfNotExists: Boolean = true): File? {
        if(accountId.isNull() || imageId.isNull() || fileNameWithExtension.isNull()){
            return null
        }
        val folder = createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.accounts}/${accountId}/${FolderName.images}/${imageId}",
            createIfNotExists
        )
        return File(folder,fileNameWithExtension)
    }

    fun getCompressedImage(accountId: String,imageId:String,fileNameWithExtension:String, createIfNotExists: Boolean = true): File? {
        if(accountId.isNull() || imageId.isNull() || fileNameWithExtension.isNull()){
            return null
        }
        val folder = createAndGetFolder(
            "${resourcesFolderPath}/${FolderName.accounts}/${accountId}/${FolderName.images}/${imageId}",
            createIfNotExists
        )
        val extension = File(fileNameWithExtension).extension
        val nameWithoutExtension = File(fileNameWithExtension).nameWithoutExtension
        return File(folder,"${nameWithoutExtension}-minifyimage.${extension}")
    }

    fun getFolderCaches(cacheName: String, createIfNotExists: Boolean = true): File? {
        return createAndGetFolder(
            "${cachesFolderPath}/${cacheName}",
            createIfNotExists
        )
    }

    fun getFolderCustomerSubscriptionInvoices(
        customerId: String,
        createIfNotExists: Boolean = true
    ): File? {
        return createAndGetFolder(
            "${resourcesFolderPath}/${customerId}/${FolderName.subscriptions}/${FolderName.invoices}/",
            createIfNotExists
        )
    }

    fun getInvoiceFile(
        customerId: String,
        invoiceId: String,
        createIfNotExists: Boolean = true
    ): File {
        return File(getFolderCustomerSubscriptionInvoices(customerId,createIfNotExists),"${invoiceId}.pdf")
    }

    private fun createAndGetFolder(path: String, createIfNotExists: Boolean = true): File? {
        return try {
            val location = File(path)
            if (createIfNotExists && !location.exists()) {
                location.mkdirs()
            }
            location
        } catch (e: Exception) {
            null
        }
    }
}